@@ -3,10 +3,10 @@ import functools
 import click
 
 from picomc.account import AccountError
-from picomc.env import Env
+from picomc.env import Env, get_filepath
 from picomc.instance import Instance
 from picomc.logging import logger
-from picomc.utils import die, get_filepath, sanitize_name
+from picomc.utils import die, sanitize_name
 
 
 def instance_list():
@@ -80,6 +80,16 @@ def launch(instance_name, account, version_override):
             logger.error("Not launching due to account error: {}".format(e))
 
 
+@instance_cli.command("natives")
+@instance_cmd
+def extract_natives(instance_name):
+    """Extract natives and leave them on disk"""
+    if not Instance.exists(instance_name):
+        die("No such instance exists.")
+    with Instance(instance_name) as inst:
+        inst.extract_natives()
+
+
 @instance_cli.command("dir")
 @instance_cmd
 def _dir(instance_name):
