@@ -4,125 +4,16 @@ import posixpath
 import shutil
 import urllib.parse
 import urllib.request
-from dataclasses import dataclass
 from functools import reduce
-from platform import architecture
-from string import Template
 
 import requests
 from picomc.downloader import DownloadQueue
-from picomc.env import Env, get_filepath
+from picomc.env import Env, file_verify_relative, get_filepath
 from picomc.javainfo import get_java_info
+from picomc.library import Library
 from picomc.logging import logger
 from picomc.rules import match_ruleset
-from picomc.utils import cached_property, die, file_sha1, file_verify_relative
-
-
-@dataclass
-class LibraryArtifact:
-    url: str
-    path: str
-    sha1: str
-    size: int
-    filename: str
-
-    @classmethod
-    def from_json(cls, obj):
-        return cls(
-            url=obj["url"],
-            path=obj["path"],
-            sha1=obj["sha1"],
-            size=obj["size"],
-            filename=None,
-        )
-
-
-class Library:
-    MOJANG_BASE_URL = "https://libraries.minecraft.net/"
-
-    def __init__(self, json_lib):
-        self.json_lib = json_lib
-        self._populate()
-
-    def _populate(self):
-        js = self.json_lib
-        self.libname = js["name"]
-        self.is_native = "natives" in js
-        self.base_url = js.get("url", Library.MOJANG_BASE_URL)
-
-        self.available = True
-
-        self.native_suffix = ""
-        if self.is_native:
-            try:
-                classifier_tmpl = self.json_lib["natives"][Env.platform]
-                arch = architecture()[0][:2]
-                self.native_classifier = Template(classifier_tmpl).substitute(arch=arch)
-                self.native_suffix = "-" + self.native_classifier
-            except KeyError:
-                logger.warn(
-                    f"Native {self.libname} is not available for current platform {Env.platform}."
-                )
-                self.native_classifier = None
-                self.available = False
-                return
-
-        self.virt_artifact = self.make_virtual_artifact()
-        self.artifact = self.resolve_artifact()
-
-        # Just use filename and path derived from the name.
-        self.filename = self.virt_artifact.url
-        self.path = self.virt_artifact.path
-
-        # Actual fs path
-        self.relpath = os.path.join(*self.path.split("/"))
-
-        if self.artifact:
-            final_art = self.artifact
-
-            # Sanity check
-            assert self.virt_artifact.path == self.artifact.path
-        else:
-            final_art = self.virt_artifact
-
-        self.url = final_art.url
-        self.sha1 = final_art.sha1
-        self.size = final_art.size
-
-    def make_virtual_artifact(self):
-        # I don't know where the *va part comes from, it was already implemented
-        # before a refactor, unfortunately the reason was not documented.
-        # Currently I don't have a version in my versions directory which
-        # utilizes that.
-        # I am leaving it implemented, as there probably was motivation to do it.
-        group, art_id, version, *va = self.libname.split(":")
-        group = group.replace(".", "/")
-        v2 = "-".join([version] + va)
-
-        filename = f"{art_id}-{v2}{self.native_suffix}.jar"
-        path = f"{group}/{art_id}/{version}/{filename}"
-        url = urllib.parse.urljoin(self.base_url, path)
-
-        return LibraryArtifact(
-            url=url, path=path, sha1=None, size=None, filename=filename
-        )
-
-    def resolve_artifact(self):
-        if self.is_native:
-            if self.native_classifier is None:
-                # Native not available for current platform
-                return None
-            else:
-                art = self.json_lib["downloads"]["classifiers"][self.native_classifier]
-                return LibraryArtifact.from_json(art)
-        else:
-            try:
-                return LibraryArtifact.from_json(self.json_lib["downloads"]["artifact"])
-            except KeyError:
-                return None
-
-    def get_abspath(self, library_root):
-        return os.path.join(library_root, self.relpath)
+from picomc.utils import cached_property, die, file_sha1
 
 
 class VersionType:
@@ -232,7 +123,6 @@ class VersionSpec:
 
 
 class Version:
-    LIBRARIES_URL = "https://libraries.minecraft.net/"
     ASSETS_URL = "http://resources.download.minecraft.net/"
 
     def __init__(self, version_name):
@@ -317,21 +207,23 @@ class Version:
             for lib in self.vspec.libraries:
                 if "rules" in lib and not match_ruleset(lib["rules"], java_info):
                     continue
-                libs.append(Library(lib))
+                lib_obj = Library(lib)
+                if not lib_obj.available:
+                    continue
+                libs.append(lib_obj)
             if key:
                 self._libraries[key] = libs
             return libs
 
-    def download_jarfile(self, force=False):
-        """Checks existence and hash of cached jar. Downloads a new one
-        if either condition is violated."""
+    def get_jarfile_dl(self, force=False):
+        """Checks existence and hash of cached jar. Returns None if ok, otherwise
+        returns download (url, size)"""
         logger.debug("Attempting to use jarfile: {}".format(self.jarfile))
         dlspec = self.vspec.downloads.get("client", None)
-        if not dlspec:
-            logger.debug("jarfile not in dlspec, skipping hash check.")
+        if dlspec is None:
+            logger.debug("jarfile dlspec not availble, skipping hash check.")
             if not os.path.exists(self.jarfile):
-                logger.error("Jarfile does not exist and can not be downloaded.")
-                raise RuntimeError()
+                die("jarfile does not exist and can not be downloaded.")
             return
 
         logger.debug("Checking jarfile.")
@@ -340,8 +232,12 @@ class Version:
             or not os.path.exists(self.jarfile)
             or file_sha1(self.jarfile) != dlspec["sha1"]
         ):
-            logger.info("Downloading jar ({}).".format(self.version_name))
-            urllib.request.urlretrieve(dlspec["url"], self.jarfile)
+            logger.info(
+                "Jar file ({}) will be downloaded with libraries.".format(
+                    self.version_name
+                )
+            )
+            return dlspec["url"], dlspec.get("size", None)
 
     def download_libraries(self, java_info, force=False):
         """Downloads missing libraries."""
@@ -361,10 +257,14 @@ class Version:
                 )
                 continue
             if force or not ok:
-                q.add(library.url, library.relpath)
+                q.add(library.url, library.get_abspath(basedir), library.size)
+        jardl = self.get_jarfile_dl()
+        if jardl is not None:
+            url, size = jardl
+            q.add(url, self.jarfile, size=size)
         if len(q) > 0:
             logger.info("Downloading {} libraries.".format(len(q)))
-        if not q.download(basedir):
+        if not q.download():
             logger.error(
                 "Some libraries failed to download. If they are part of a non-vanilla profile, the original installer may need to be used."
             )
@@ -392,9 +292,9 @@ class Version:
     def download_assets(self, force=False):
         """Downloads missing assets."""
 
-        hashes = set()
+        hashes = dict()
         for obj in self.raw_asset_index["objects"].values():
-            hashes.add(obj["hash"])
+            hashes[obj["hash"]] = obj["size"]
 
         logger.info("Checking {} assets.".format(len(hashes)))
 
@@ -406,11 +306,11 @@ class Version:
             if file_verify_relative(path, sha):
                 continue
             url = urllib.parse.urljoin(self.ASSETS_URL, posixpath.join(sha[0:2], sha))
-            q.add(url, path)
+            q.add(url, os.path.join(Env.app_root, path), size=hashes[sha])
 
         if len(q) > 0:
             logger.info("Downloading {} assets.".format(len(q)))
-        if not q.download(Env.app_root):
+        if not q.download():
             logger.error("Some assets failed to download.")
 
         if is_virtual:
@@ -422,7 +322,6 @@ class Version:
     def prepare(self, java_info=None):
         if not java_info:
             java_info = get_java_info(Env.gconf.get("java.path"))
-        self.download_jarfile()
         self.download_libraries(java_info)
         self.download_assets()
 
@@ -477,7 +376,7 @@ class VersionManager:
         if local:
             import os
 
-            r += (
+            r += sorted(
                 "{} [local]".format(name)
                 for name in os.listdir(get_filepath("versions"))
                 if os.path.isdir(get_filepath("versions", name))
